"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const puppeteer = require("puppeteer");
const Logger = require("log4js");
const P = require("bluebird");
let _logger = Logger.getLogger("shot-worker");
class ShotWorker {
    constructor(id) {
        this.id = id;
        this.shotCallback = _.noop;
        this.isBusy = true;
    }
    static create(idx, config) {
        return __awaiter(this, void 0, void 0, function* () {
            let worker = new ShotWorker(idx);
            worker.init(idx, config.callbackName, config.warmerUrl, config.width, config.height);
            return worker;
        });
    }
    takeShot(url) {
        if (this.isBusy) {
            _logger.error('Worker is busy doing work');
            return P.reject('Worker is already busy');
        }
        let start = (new Date()).valueOf();
        _logger.debug('screenshot url', url);
        return new P((resolve, reject) => {
            this.shotCallback = (err, buffer) => {
                if (err) {
                    return reject(err);
                }
                resolve(buffer);
            };
            this.page.goto(url, { waitUntil: 'networkidle' });
        });
    }
    reload() {
        return this.page.reload();
    }
    exit() {
        this.browser && this.browser.close();
    }
    init(idx = 0, callbackName = 'callPhantom', warmerUrl = '', width = 800, height = 600) {
        return __awaiter(this, void 0, void 0, function* () {
            let start = (new Date()).valueOf();
            this.browser = yield puppeteer.launch({
                ignoreHTTPSErrors: true,
                headless: true,
                args: ['--ignore-certificate-errors'],
                userDataDir: '/tmp/chrome'
            });
            this.page = yield this.browser.newPage();
            this.page.on('console', (...args) => _logger.debug('PAGE LOG:', ...args));
            this.page.on('response', r => _logger.debug(r.status + ' ' + r.url));
            this.page.setViewport({
                width: width,
                height: height
            });
            // Define a window.onCustomEvent function on the page.  
            yield this.page.exposeFunction(callbackName, e => {
                _logger.debug('Callback called from browser with', e);
                return this.page.screenshot({
                    fullPage: true
                }).then((buffer) => {
                    this.shotCallback(null, buffer);
                    this.isBusy = false;
                }, (err) => {
                    this.shotCallback(err, null);
                    this.isBusy = false;
                });
            });
            if (warmerUrl) {
                yield this.page.goto(warmerUrl, { waitUntil: 'networkidle' });
            }
            _logger.info(`Worker ${idx} ready`);
            this.creationTime = (new Date()).valueOf() - start;
            this.isBusy = false;
        });
    }
}
exports.ShotWorker = ShotWorker;
//# sourceMappingURL=shot-worker.js.map